% This file will convert DeepLabCut csv output files into a format that Ty
% Hedrick's DLTdv software can convert to 3D coordinate csvs.

% Prompting user to select number of video views
prompt = {'How many separate cameras do you have?','How many points did you track?'};
input = inputdlg(prompt);
cams = str2num(input{1,1}); % convert string input to integer
points = str2num(input{2,1}); % convert string input to integer

% Get the pixel coordinates from the cameras you want (make sure it is in order you calibrated!)
for i = 1:cams
    title = sprintf('Select DLC output csv number %d',i);
    [cam{i,1},cam_path{i,1}] = uigetfile('*.csv',title);
    opts = detectImportOptions([cam_path{i,1},cam{i,1}],'NumHeaderLines',1); %number of headers to be ignored
    opts.VariableNamesLine = 2; % row of variable (point) name
    opts.VariableUnitsLine = 3; % NOT units but specify x or y axis of pixels
    DLC_xy{i,1} = readtable([cam_path{i,1},cam{i,1}],opts);
    DLC_xy{i,1}(:,1) = []; % delete first column which just gives frame number
    
    % getting rid of likelyhood columns
    cols = size(DLC_xy{i,1},2);
    for j = 0:3:(cols-3)
        DLC_xy{i,1}(:,cols-j) = []; % deletes the likelyhood columns from right to left
    end    
end

% Putting DeepLabCut data in table format for DLTdv software
for n = 1:points
    for i = 1:cams
        DLT_x{1,n}(:,i) = DLC_xy{i,1}{:,(2*n-1)};
        DLT_y{1,n}(:,i) = DLC_xy{i,1}{:,(2*n)};
    end
end

% Create final DLT_xy Table
DLT_xy_array = [DLT_x{1,1}(:,1), DLT_y{1,1}(:,1)]; % first point from first camera
% append remaining columns to table
for n = 1:points
    if n == 1
        for i = 2:cams % skip over first view for 1st point since already done
            DLT_xy_array = [DLT_xy_array, DLT_x{1,1}(:,i), DLT_y{1,1}(:,i)]; % add in incremental x & y coordinates for all the views
        end
    else
        for i = 1:cams
            DLT_xy_array = [DLT_xy_array, DLT_x{1,n}(:,i), DLT_y{1,n}(:,i)]; % add in incremental x & y coordinates for all the views
        end
    end
end
% Creating Table Headers
name = 1;
for n = 1:points
    for i = 1:cams
        VarNames{1,name} = ['pt' num2str(n) '_cam' num2str(i) '_X'];
        name = name + 1; % move to next name value
        VarNames{1,name} = ['pt' num2str(n) '_cam' num2str(i) '_Y'];
        name = name + 1; % move to next name value
    end
end
DLT_xy = array2table(DLT_xy_array,'VariableNames',VarNames);

%% Creating blank XYZ DLT file
name2 = 1;
for n = 1:points
   VarNames2{1,name2} = ['Pt' num2str(n) '_X'];
   name2 = name2 + 1;
   VarNames2{1,name2} = ['Pt' num2str(n) '_Y'];
   name2 = name2 + 1;
   VarNames2{1,name2} = ['Pt' num2str(n) '_Z'];
   name2 = name2 + 1;
end
DLT_xyz_array = nan(size(DLT_xy,1),(points*3));
DLT_xyz = array2table(DLT_xyz_array,'VariableNames',VarNames2);

%% Creating blank XYZres DLT file
for n = 1:points
    VarNames3{1,n} = ['pt' num2str(n) '_dltres'];
end
DLT_xyzres_array = nan(size(DLT_xy,1),points);
DLT_xyzres = array2table(DLT_xyzres_array,'VariableNames',VarNames3);

%% Creating offsets file
% ASSUMES that there is no offset between the multiple cameras.
DLT_offsets_array = zeros(size(DLT_xy,1),cams);
for i = 1:cams
    VarNames4{1,i} = ['cam' num2str(i) '_offset'];
end
DLT_offsets = array2table(DLT_offsets_array,'VariableNames',VarNames4);

%% Saving files to desired location
% Have user select the directory they would like to save the files
dirPrompt = 'Choose the folder you want to save the DLTdv files.';
savePath = uigetdir(cd,dirPrompt);

% Get base filename to save all necessary files
prompt2 = 'Enter the base filename you want to save the four DLT files with:';
filebase = inputdlg(prompt2);

% Save all four files to specified directory
filename{1,1} = strcat(savePath, filesep, filebase, '_xypts.csv');
filename{1,2} = strcat(savePath, filesep, filebase, '_xyzpts.csv');
filename{1,3} = strcat(savePath, filesep, filebase, '_xyzres.csv');
filename{1,4} = strcat(savePath, filesep, filebase, '_offsets.csv');
writetable(DLT_xy,filename{1,1}{1,:});
writetable(DLT_xyz,filename{1,2}{1,:});
writetable(DLT_xyzres,filename{1,3}{1,:});
writetable(DLT_offsets,filename{1,4}{1,:});